@@ -1,11 +1,17 @@
-// Copyright (C) 2008-2015 National ICT Australia (NICTA)
+// Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
+// Copyright 2008-2016 National ICT Australia (NICTA)
 // 
-// This Source Code Form is subject to the terms of the Mozilla Public
-// License, v. 2.0. If a copy of the MPL was not distributed with this
-// file, You can obtain one at http://mozilla.org/MPL/2.0/.
-// -------------------------------------------------------------------
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+// http://www.apache.org/licenses/LICENSE-2.0
 // 
-// Written by Conrad Sanderson - http://conradsanderson.id.au
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+// ------------------------------------------------------------------------
 
 
 //! \addtogroup unwrap
@@ -198,7 +204,7 @@ struct quasi_unwrap_fixed
     arma_extra_debug_sigprint();
     }
   
-  const Mat<eT>& M;
+  const T1& M;
   
   template<typename eT2>
   arma_inline bool is_alias(const Mat<eT2>& X) const { return (void_ptr(&M) == void_ptr(&X)); }
@@ -295,6 +301,28 @@ struct quasi_unwrap< Col<eT> >
 
 
 
+template<typename eT>
+struct quasi_unwrap< subview<eT> >
+  {
+  static const bool has_subview = true;
+  
+  inline
+  quasi_unwrap(const subview<eT>& A)
+    : sv( A                  )
+    , M ( A, (A.n_cols == 1) )  // reuse memory if the subview has only one column
+    {
+    arma_extra_debug_sigprint();
+    }
+  
+  const subview<eT>& sv;
+  const Mat<eT>      M;
+  
+  template<typename eT2>
+  arma_inline bool is_alias(const Mat<eT2>& X) const { return ( (sv.n_cols == 1) ? (void_ptr(&(sv.m)) == void_ptr(&X)) : false ); }
+  };
+
+
+
 template<typename eT>
 struct quasi_unwrap< subview_row<eT> >
   {
@@ -310,7 +338,7 @@ struct quasi_unwrap< subview_row<eT> >
   const Row<eT> M;
   
   template<typename eT2>
-  arma_inline bool is_alias(const Mat<eT2>& X) const { return (void_ptr(&M) == void_ptr(&X)); }
+  arma_inline bool is_alias(const Mat<eT2>&) const { return false; }
   };
 
 
